-- 粒子系统
-- 支持重力粒子

CParticleSystem = {};
function CParticleSystem.new( tblConfig )
    local self = {};
    setmetatable( self, CParticleSystem );
    CParticleSystem.__index = CParticleSystem;

    local vStartColor =
    {
        tblConfig.startColorRed,
        tblConfig.startColorGreen,
        tblConfig.startColorBlue,
        tblConfig.startColorAlpha
    };
    local vStartColorVariance =
    {
        tblConfig.startColorVarianceRed,
        tblConfig.startColorVarianceGreen,
        tblConfig.startColorVarianceBlue,
        tblConfig.startColorVarianceAlpha,
    };
    local vFinishColor =
    {
        tblConfig.finishColorRed,
        tblConfig.finishColorGreen,
        tblConfig.finishColorBlue,
        tblConfig.finishColorAlpha
    };
    local vFinishColorVariance =
    {
        tblConfig.finishColorVarianceRed,
        tblConfig.finishColorVarianceGreen,
        tblConfig.finishColorVarianceBlue,
        tblConfig.finishColorVarianceAlpha,
    };

    self.m_nMaxParticles            = tblConfig.maxParticles;               -- 最大粒子数
    self.m_xGravity                 = tblConfig.gravityx;                   -- 重力x方向
    self.m_yGravity                 = tblConfig.gravityy;                   -- 重力y方向
    self.m_fAngle                   = tblConfig.angle;                      -- 初始力方向
    self.m_fAngleVariance           = tblConfig.angleVariance;              -- 初始力方向方差
    self.m_fSpeed                   = tblConfig.speed;                      -- 初始速度
    self.m_fSpeedVariance           = tblConfig.speedVariance;              -- 初始速度方差
    self.m_vStartColor              = vStartColor;                          -- 开始颜色
    self.m_vStartColorVariance      = vStartColorVariance;                  -- 开始颜色方差
    self.m_vFinishColor             = vFinishColor;                         -- 结束颜色
    self.m_vFinishColorVariance     = vFinishColorVariance;                 -- 结束颜色方差
    self.m_fStartSize               = tblConfig.startParticleSize;          -- 开始大小
    self.m_fStartSizeVariance       = tblConfig.startParticleSizeVariance;  -- 开始大小方差
    self.m_fFinishSize              = tblConfig.finishParticleSize;         -- 结束大小
    self.m_fFinishSizeVariance      = tblConfig.finishParticleSizeVariance; -- 结束大小方差
    self.m_fStartRotation           = tblConfig.rotationStart;              -- 粒子开始旋转
    self.m_fStartRotationVariance   = tblConfig.rotationStartVariance;      -- 粒子开始旋转方差
    self.m_fFinishRotation          = tblConfig.rotationEnd;                -- 粒子结束旋转
    self.m_fFinishRotationVariance  = tblConfig.rotationEndVariance;        -- 粒子结束旋转方差
    self.m_fLifeSpan                = tblConfig.particleLifespan;           -- 存活时间
    self.m_fLifeSpanVariance        = tblConfig.particleLifespanVariance;   -- 存活时间方差
    self.m_xPosition                = tblConfig.sourcePositionx;            -- 出生位置x
    self.m_yPosition                = tblConfig.sourcePositiony;            -- 出生位置y
    self.m_xPositionVariance        = tblConfig.sourcePositionVariancex;    -- 出生位置x方差
    self.m_yPositionVariance        = tblConfig.sourcePositionVariancey;    -- 出生位置y方差
    self.m_fRadialAcceleration      = tblConfig.radialAcceleration;         -- 径向加速度
    self.m_fRadialAccelVariance     = tblConfig.radialAccelVariance;        -- 径向加速度方差
    self.m_fTangentialAcceleration  = tblConfig.tangentialAcceleration;     -- 切向加速度
    self.m_fTangentialAccelVariance = tblConfig.tangentialAccelVariance;    -- 切向加速度方差

    self.m_fEmissionRate = self.m_nMaxParticles / self.m_fLifeSpan;
    self.m_tblParticle = {};
    self.m_nParticleCount = 0;
    self.m_nEmitCounter = 0;
    self.m_nUpdateTime = 0;

    for i = 1, self.m_nMaxParticles do
        self.m_tblParticle[i] =
        {
            fLifeTime           = 0,
            xPos                = 0,
            yPos                = 0,
            fRadialAccel        = 0,
            fTangentialAccel    = 0,
            xDir                = 0,
            yDir                = 0,
            fSize               = 0,
            fDeltaSize          = 0,

            rColor              = 0,
            gColor              = 0,
            bColor              = 0,
            aColor              = 0,
            rDeltaColor         = 0,
            gDeltaColor         = 0,
            bDeltaColor         = 0,
            aDeltaColor         = 0,

            fRotation           = 0,
            fDeltaRotation      = 0,
        };
    end

    self:Init();
    return self;
end

function CParticleSystem:Init()
    self.m_nParticleCount   = 0;
    self.m_nEmitCounter     = 0;
    self.m_nUpdateTime      = 0;
end

function CParticleSystem:Random()
    return math.random() - 0.5;
end

function CParticleSystem:AddParticle( nCount )
    local nCurCount = self.m_nParticleCount;
    for i = 1, nCount do
        local fLifeTime = self.m_fLifeSpan + self.m_fLifeSpanVariance * self:Random();
        local fStartSize = self.m_fStartSize + self.m_fStartSizeVariance * self:Random();
        local fEndSize = self.m_fFinishSize + self.m_fFinishSizeVariance * self:Random();
        local fSpeed = self.m_fSpeed + self.m_fSpeedVariance * self:Random();
        local fAngle = self.m_fAngle + self.m_fAngleVariance * self:Random();
        local rStartColor = self.m_vStartColor[1] + self.m_vStartColorVariance[1] * self:Random();
        local gStartColor = self.m_vStartColor[2] + self.m_vStartColorVariance[2] * self:Random();
        local bStartColor = self.m_vStartColor[3] + self.m_vStartColorVariance[3] * self:Random();
        local aStartColor = self.m_vStartColor[4] + self.m_vStartColorVariance[4] * self:Random();
        local rFinishColor = self.m_vFinishColor[1] + self.m_vFinishColorVariance[1] * self:Random();
        local gFinishColor = self.m_vFinishColor[2] + self.m_vFinishColorVariance[2] * self:Random();
        local bFinishColor = self.m_vFinishColor[3] + self.m_vFinishColorVariance[3] * self:Random();
        local aFinishColor = self.m_vFinishColor[4] + self.m_vFinishColorVariance[4] * self:Random();
        local fStartRotation = self.m_fStartRotation + self.m_fStartRotationVariance * self:Random();
        local fFinishRotation = self.m_fFinishRotation + self.m_fFinishRotationVariance * self:Random();

        self.m_nParticleCount = self.m_nParticleCount + 1;
        local tbl = self.m_tblParticle[nCurCount + i];
        tbl.fLifeTime = fLifeTime;
        tbl.xPos = self.m_xPosition + self.m_xPositionVariance * self:Random();
        tbl.yPos = self.m_yPosition + self.m_yPositionVariance * self:Random();
        tbl.fRadialAccel = self.m_fRadialAcceleration + self.m_fRadialAccelVariance * self:Random();
        tbl.fTangentialAccel = self.m_fTangentialAcceleration + self.m_fTangentialAccelVariance * self:Random();
        tbl.xDir = math.cos( fAngle / 180 * math.pi ) * fSpeed;
        tbl.yDir = math.sin( fAngle / 180 * math.pi ) * fSpeed;
        tbl.fSize = fStartSize;
        tbl.fDeltaSize = ( fEndSize - fStartSize ) / fLifeTime;

        tbl.rColor = rStartColor;
        tbl.gColor = gStartColor;
        tbl.bColor = bStartColor;
        tbl.aColor = aStartColor;
        tbl.rDeltaColor = ( rFinishColor - rStartColor ) / fLifeTime;
        tbl.gDeltaColor = ( gFinishColor - gStartColor ) / fLifeTime;
        tbl.bDeltaColor = ( bFinishColor - bStartColor ) / fLifeTime;
        tbl.aDeltaColor = ( aFinishColor - aStartColor ) / fLifeTime;

        tbl.fRotation = fStartRotation;
        tbl.fDeltaRotation = (fFinishRotation - fStartRotation) / fLifeTime;
    end
end

local g_nInterval = 33;
local g_nIntervalS = g_nInterval / 1000;
function CParticleSystem:Update( nDeltaTime )
    local nTolTime = self.m_nUpdateTime + nDeltaTime;
    local nTimeCount = math.floor( nTolTime / g_nInterval );
    self.m_nUpdateTime = nTolTime - nTimeCount * g_nInterval;
    nDeltaTime = g_nIntervalS;

    for i = 1, nTimeCount do
        local fRate = 1.0 / self.m_fEmissionRate;
        local nTotalParticles = self.m_nMaxParticles;
        local nParticleCount = self.m_nParticleCount;
        if nParticleCount < nTotalParticles then
            self.m_nEmitCounter = math.max( self.m_nEmitCounter + nDeltaTime, 0 );
        end
        local nEmitCount = math.min( nTotalParticles - nParticleCount, math.floor( self.m_nEmitCounter / fRate ) );
        self:AddParticle( nEmitCount );
        self.m_nEmitCounter = self.m_nEmitCounter - fRate * nEmitCount;

        for i = self.m_nParticleCount, 1, -1 do
            local tblParticle = self.m_tblParticle[i];
            tblParticle.fLifeTime = tblParticle.fLifeTime - nDeltaTime;
            if tblParticle.fLifeTime <= 0 then
                self.m_tblParticle[i] = self.m_tblParticle[self.m_nParticleCount];
                self.m_tblParticle[self.m_nParticleCount] = tblParticle;
                self.m_nParticleCount = self.m_nParticleCount - 1;
            end
        end

        for i = 1, self.m_nParticleCount do
            local tblParticle = self.m_tblParticle[i];

            -- 计算重力位置
            local xPos = tblParticle.xPos;
            local yPos = tblParticle.yPos;
            local fPosLen = math.sqrt( xPos * xPos + yPos * yPos );
            local xRadial = 0;
            local yRadial = 0;
            if fPosLen > 0 then
                xRadial = xPos / fPosLen;
                yRadial = yPos / fPosLen;
            end
            -- 先交换x，y
            local xTangential = yRadial;
            local yTangential = xRadial;
            xRadial = xRadial * tblParticle.fRadialAccel;
            yRadial = yRadial * tblParticle.fRadialAccel;
            xTangential = xTangential * (-tblParticle.fTangentialAccel);
            yTangential = yTangential * ( tblParticle.fTangentialAccel);
            local xTemp = xRadial + xTangential + self.m_xGravity;
            local yTemp = yRadial + yTangential + self.m_yGravity;
            xTemp = xTemp * nDeltaTime;
            yTemp = yTemp * nDeltaTime;
            tblParticle.xDir = tblParticle.xDir + xTemp;
            tblParticle.yDir = tblParticle.yDir + yTemp;
            xTemp = tblParticle.xDir * nDeltaTime;
            yTemp = tblParticle.yDir * nDeltaTime;
            tblParticle.xPos = tblParticle.xPos + xTemp;
            tblParticle.yPos = tblParticle.yPos + yTemp;

            tblParticle.fSize = math.max( 0, tblParticle.fSize + tblParticle.fDeltaSize * nDeltaTime );
            tblParticle.rColor = tblParticle.rColor + tblParticle.rDeltaColor * nDeltaTime;
            tblParticle.gColor = tblParticle.gColor + tblParticle.gDeltaColor * nDeltaTime;
            tblParticle.bColor = tblParticle.bColor + tblParticle.bDeltaColor * nDeltaTime;
            tblParticle.aColor = tblParticle.aColor + tblParticle.aDeltaColor * nDeltaTime;
            tblParticle.fRotation = tblParticle.fRotation + tblParticle.fDeltaRotation * nDeltaTime;
        end
    end
end

function CParticleSystem:GetParticleCount()
    return self.m_nParticleCount;
end

function CParticleSystem:GetPosX( i )
    return self.m_tblParticle[i].xPos;
end

function CParticleSystem:GetPosY( i )
    return self.m_tblParticle[i].yPos;
end

function CParticleSystem:GetRotation( i )
    return self.m_tblParticle[i].fRotation;
end

function CParticleSystem:GetSize( i )
    return self.m_tblParticle[i].fSize;
end

function CParticleSystem:GetColorA( i )
    return self.m_tblParticle[i].aColor;
end

function CParticleSystem:GetColorR( i )
    return self.m_tblParticle[i].rColor;
end

function CParticleSystem:GetColorG( i )
    return self.m_tblParticle[i].gColor;
end

function CParticleSystem:GetColorB( i )
    return self.m_tblParticle[i].bColor;
end