MTVideo =
{
    m_tblCrea = nil;
    m_pDecode = nil,
    m_pSample = nil;
    m_fTotTime = 0;
    m_fCurTime = 0;
    m_fGetTime = -1;
    m_nFps = 0;
};
function MTVideo.new( szPath, nVideoType, tblInfo, nFps )
    local self = {};
    setmetatable( self, MTVideo );
    MTVideo.__index = MTVideo;

    self.m_nFps = nFps;
    self.m_tblCrea = { szPath, nVideoType, tblInfo, #tblInfo, nFps, false, true, true };
    self:Reinit();
    
    local nWidth = self.m_pDecode:GetWidth();
    local nDepth = self.m_pDecode:GetHeight();
    local pTexture = Texture.create( Texture.RGBA8888, nWidth,
        nDepth, {}, false, Texture.TEXTURE_2D );
    self.m_pSample = Texture.Sampler.create( pTexture );
	self.m_pSample:setWrapMode( Texture.CLAMP, Texture.CLAMP, Texture.CLAMP );
    self.m_fTotTime = self.m_pDecode:GetVideoTotalTime();
    return self;
end

function MTVideo:Reinit()
    local tbl = self.m_tblCrea;
    self.m_pDecode = MVideoDecoder.new( tbl[1], tbl[2],
        tbl[3], tbl[4], tbl[5], tbl[6], tbl[7], tbl[8] );
end

function MTVideo:Reset()
    --  强制获取最后一帧空白帧
    self.m_fCurTime = self.m_fTotTime - 1;
    self:GetSampler();

    self.m_fCurTime = 0;
end

function MTVideo:Update( nDeltaTime )
    self.m_fCurTime = ( self.m_fCurTime + nDeltaTime ) % self.m_fTotTime;
end

function MTVideo:GetSampler()
    if self.m_fGetTime == self.m_fCurTime then
        return self.m_pSample;
    end

    self.m_fGetTime = self.m_fCurTime;
    self.m_pDecode:LoadToSampler( self.m_fCurTime, self.m_pSample );
    return self.m_pSample;
end

function MTVideo:GetCurTime()
    return self.m_fCurTime;
end

function MTVideo:GetTolTime()
    return self.m_fTotTime;
end

function MTVideo:GetFps()
    return self.m_nFps;
end

MTVideoAlpha = {};
function MTVideoAlpha.new( nVideoType, tblInfo, nFps, aryVideoPath )
    local self = {};
    setmetatable( self, MTVideoAlpha );
    MTVideoAlpha.__index = MTVideoAlpha;

    local pVideoColo = MTVideo.new( aryVideoPath[1], nVideoType, tblInfo, nFps );
    if #aryVideoPath == 1 then
        self.m_pVideoColo = pVideoColo;
        self.m_pVideoMask = pVideoColo;
        self.m_vColoMat = Vector4.new( 1.0, 0.5, 0.0, 0.0 );
        self.m_vMaskMat = Vector4.new( 1.0, 0.5, 0.0, 0.5 );
    else
        self.m_pVideoColo = pVideoColo;
        self.m_pVideoMask = MTVideo.new( aryVideoPath[2], nVideoType, tblInfo, nFps );
        self.m_vColoMat = Vector4.new( 1.0, 1.0, 0.0, 0.0 );
        self.m_vMaskMat = Vector4.new( 1.0, 1.0, 0.0, 0.0 );
    end

    if self.m_pVideoColo:GetTolTime() ~= self.m_pVideoMask:GetTolTime() then
        return nil;
    end

    return self;
end

function MTVideoAlpha:GetColoSampler()
    return self.m_pVideoColo:GetSampler();
end

function MTVideoAlpha:GetMaskSampler()
    return self.m_pVideoMask:GetSampler();
end

function MTVideoAlpha:GetColoMat()
    return self.m_vColoMat;
end

function MTVideoAlpha:GetMaskMat()
    return self.m_vMaskMat;
end

function MTVideoAlpha:Update( nDeltaTime )
    self.m_pVideoColo:Update( nDeltaTime );
    if self.m_pVideoColo ~= self.m_pVideoMask then
        self.m_pVideoMask:Update( nDeltaTime );
    end
end

function MTVideoAlpha:Reset()
    self.m_pVideoColo:Reset();
    if self.m_pVideoColo ~= self.m_pVideoMask then
        self.m_pVideoMask:Reset();
    end
end

function MTVideoAlpha:GetTolTime()
    return self.m_pVideoColo:GetTolTime();
end

function MTVideoAlpha:GetCurTime()
    return self.m_pVideoColo:GetCurTime();
end